@@ -588,17 +588,17 @@ NET_API_STATUS NetrShareEnum(
     case 2:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level2);
-        InfoStruct->ShareInfo.Level2->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_2));
+        InfoStruct->ShareInfo.Level2->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_2));
         break;
     case 1:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level1);
-        InfoStruct->ShareInfo.Level1->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_1));
+        InfoStruct->ShareInfo.Level1->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_1));
         break;
     case 0:
         if (space_limited)
             space_available -= sizeof(InfoStruct->ShareInfo.Level0);
-        InfoStruct->ShareInfo.Level0->Buffer = calloc((enump->count - enump->next), sizeof(SHARE_INFO_0));
+        InfoStruct->ShareInfo.Level0->Buffer = MIDL_user_allocate((enump->count - enump->next) * sizeof(SHARE_INFO_0));
         break;
     }
 
@@ -735,22 +735,7 @@ NET_API_STATUS NetrShareGetInfo(
 
     dscp = cm_RootSCachep(userp, &req);
 
-    /* Allocate the memory for the response */
-    switch (Level) {
-    case 2:
-        InfoStruct->ShareInfo2 = calloc(1, sizeof(SHARE_INFO_2));
-        break;
-    case 1:
-        InfoStruct->ShareInfo1 = calloc(1, sizeof(SHARE_INFO_1));
-        break;
-    case 0:
-        InfoStruct->ShareInfo0 = calloc(1, sizeof(SHARE_INFO_0));
-        break;
-    }
-
-    if (InfoStruct->ShareInfo0 == NULL) {
-        return ERROR_NOT_ENOUGH_MEMORY;
-    }
+    InfoStruct->ShareInfo0 = NULL;
 
     /*
      * NetName will be:
@@ -810,7 +795,10 @@ NET_API_STATUS NetrShareGetInfo(
         if (allSubmount) {
             scp = dscp;
             cm_HoldSCache(scp);
-        }
+	    code = 0;
+	} else {
+	    code = CM_ERROR_NOSUCHPATH;
+	}
     } else {
         /*
          * Could be a Submount, a directory entry, or a cell name we
@@ -845,11 +833,40 @@ NET_API_STATUS NetrShareGetInfo(
                     code == CM_ERROR_BPLUS_NOMATCH)
                     code = cm_NameI(dscp, pathstr, CM_FLAG_CASEFOLD | CM_FLAG_FOLLOW,
                                     userp, NULL, &req, &scp);
-            }
+	    } else {
+		code = CM_ERROR_NOSUCHPATH;
+	    }
         }
     }
 
     if (scp) {
+	/* Allocate the memory for the response */
+	switch (Level) {
+	case 2:
+	    InfoStruct->ShareInfo2 = MIDL_user_allocate(sizeof(SHARE_INFO_2));
+	    break;
+	case 1:
+	    InfoStruct->ShareInfo1 = MIDL_user_allocate(sizeof(SHARE_INFO_1));
+	    break;
+	case 0:
+	    InfoStruct->ShareInfo0 = MIDL_user_allocate(sizeof(SHARE_INFO_0));
+	    break;
+	case 501:
+	case 502:
+	case 503:
+	case 1004:
+	case 1005:
+	case 1006:
+	default:
+	    cm_ReleaseSCache(scp);
+	    return HRESULT_FROM_WIN32(ERROR_INVALID_LEVEL);
+	}
+
+	if (InfoStruct->ShareInfo0 == NULL) {
+	    cm_ReleaseSCache(scp);
+	    return HRESULT_FROM_WIN32(ERROR_NOT_ENOUGH_MEMORY);
+	}
+
         switch (Level) {
         case 2:
             /* for share level security */
@@ -952,16 +969,16 @@ NET_API_STATUS NetrServerGetInfo(
      */
     switch (Level) {
     case 103:
-        InfoStruct->ServerInfo103 = calloc(1, sizeof(SERVER_INFO_103));
+        InfoStruct->ServerInfo103 = MIDL_user_allocate(sizeof(SERVER_INFO_103));
         break;
     case 102:
-        InfoStruct->ServerInfo102 = calloc(1, sizeof(SERVER_INFO_102));
+        InfoStruct->ServerInfo102 = MIDL_user_allocate(sizeof(SERVER_INFO_102));
         break;
     case 101:
-        InfoStruct->ServerInfo101 = calloc(1, sizeof(SERVER_INFO_101));
+        InfoStruct->ServerInfo101 = MIDL_user_allocate(sizeof(SERVER_INFO_101));
         break;
     case 100:
-        InfoStruct->ServerInfo100 = calloc(1, sizeof(SERVER_INFO_100));
+        InfoStruct->ServerInfo100 = MIDL_user_allocate(sizeof(SERVER_INFO_100));
         break;
     }
 
